@@ -1,19 +1,19 @@
 BUILD_BIT='32bit'
 BUILD_CPU='ARMv7 Processor rev 3 (v7l)'
-BUILD_DATE='2020-02-18'
+BUILD_DATE='2020-04-05'
 BUILD_MACHINE='armv7l'
 BUILD_OS='Linux unknown unknown unknown'
-BUILD_TIME='10:17'
+BUILD_TIME='20:42'
 COMPILER_VERSION='armv7l-unknown-linux-gnueabihf-gcc (GCC) 9.2.0'
-CYTHON_VERSION='0.29.15'
+CYTHON_VERSION='0.29.16'
 LINKER_VERSION='GNU ld (GNU Binutils) 2.33.1'
-PYTHON_VERSION='3.8.1'
+PYTHON_VERSION='3.8.2'
 RELEASE_BUILD=True
-lib_gtk='3.24.14'
+lib_gtk='3.24.17'
 lib_vpx='1.8.2'
 lib_x11='1.6.9'
 lib_x264='0.159.2991'
-lib_x265='3.2.1'
+lib_x265='3.3'
 lib_xcomposite='0.4.5'
 lib_xdamage='1.1.5'
 lib_xext='1.3.4'
